import contextlib
import multiprocessing
from functools import wraps
from multiprocessing.pool import IMapIterator

from outpostkit.repository.lfs.logger import create_lfs_logger

_log = create_lfs_logger(__name__)
@contextlib.contextmanager
def multimap(cores=None):
    """
    Provide multiprocessing imap like function.
    The context manager handles setting up the pool, worked around interrupt issues
    and terminating the pool on completion.
    """
    if cores is None:
        cores = max(multiprocessing.cpu_count() - 1, 1)

    def wrapper(func):
        def wrap(self, timeout=None):
            return func(self, timeout=timeout if timeout is not None else 1e100)

        return wrap

    IMapIterator.next = wrapper(IMapIterator.next)
    pool = multiprocessing.Pool(cores)
    yield pool.imap
    pool.terminate()


def map_wrap(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        _log.info(args, kwargs)
        return f(*args, **kwargs)

    return wrapper
